package PerlLib::Util;

use Manager::Dialog qw (ApproveStrongly SubsetSelect QueryUser ApproveCommands );

use Data::Dumper;
use Error qw(:try);
use File::Stat;
use String::Similarity;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw ( LookupSimilar Date SaveDataToFile ExistsRecent DeDumper PidsForProcess KillProcesses );

sub LookupSimilar {
  my (%args) = (@_);
  my ($limit,$similarity,@matches) = (0,0,());
  my $depth = $args{Depth} || 15;

  if (0) {
    foreach my $key (keys %args){
      print "<$key $args{$key}>\n";
    }
  }

  if ($args{Function}) {
    foreach my $item (@{$args{Candidates}}) {
      $candidate =  &{$args{Function}}($item);
      $similarity = similarity($args{Query},$candidate,0);
      push @matches, [$candidate, sprintf("%0.5f",$similarity)];
    }
  } else {
    foreach my $candidate (@{$args{Candidates}}) {
      $similarity = similarity($args{Query},$candidate,0);
      push @matches, [$candidate, sprintf("%0.5f",$similarity)];
    }
  }

  my @tmp = sort {$b->[1] <=> $a->[1]} @matches;
  my @sorted = splice(@tmp,0,$depth);
  my @items = map {"<<<" . $_->[1] . ">\t<" . $_->[0] . ">>>"} @sorted;
  my @ans = SubsetSelect(Set => \@items,
			 Selection => {},
			 Type => "int");
  foreach my $ans (@ans) {
    return $sorted[$ans]->[0];
  }
}

sub Date {
  my (%args) = @_;
  my $date = `date "+%Y%m%d%H%M%S"`;
  chomp $date;
  return $date;
}

sub SaveDataToFile {
  my %args = @_;
  my $f = $args{File};
  my $ok = 0;
  if (-e $f) {
    if (ApproveStrongly("Overwrite file <$f>?")) {
      $ok = 1;
    }
  } else {
    $ok = 1;
  }
  if ($ok) {
    my $OUT;
    if ($args{Append}) {
      open(OUT,">>$f") or die "Cannot open file $f\n";
    } else {
      open(OUT,">$f")  or die "Cannot open file $f\n";
    }
    print OUT $args{Data};
    close(OUT);
  }
}

sub ExistsRecent {
  my %args = @_;
  my $f = $args{File};
  my $r = {};
  if (-f $f) {
    $r->{Exists} = 1;
    my $stat = File::Stat->new($f);
    my $diff = time - $stat->mtime;
    if ($diff) {
      if ($diff < $args{Within}) {
	$r->{Recent} = 1;
      } else {
	$r->{Recent} = 0;
      }
    }
  } else {
    $r->{Exists} = 0;
  }
  return $r;
}

sub RandomizeList {
  my @m = @_;
  my @l;
  while (@m) {
    my $i = int(rand(scalar @m));
    push @l, $m[$i];
    $m[$i] = $m[$#m];
    pop @m;
  }
  return @l;
}


sub DeDumper {
  my $item = shift;
  my $it;
  try {
    $VAR1 = undef;
    eval $item;
    $it = $VAR1;
    $VAR1 = undef;
  }
    catch Error with {
      print Dumper({Problem => (shift)});
    };
  return $it;
}

sub DeDumperOld {
  my $item = shift;
  $VAR1 = undef;
  # print Dumper($item);
  eval $item;
  my $it = $VAR1;
  $VAR1 = undef;
  return $it;
}

sub PidsForProcess {
  my %args = @_;
  my @pids;
  my $sample1 = "andrewdo 11701  0.2 27.7 1505288 1071032 pts/8 Sl+  01:07   1:47 ";
  my $length1 = length($sample1);
  foreach my $line (split /\n/, `ps aux --cols=300`) {
    my $process = $line;
    $process =~ s/^.{$length1}//;
    if ($process eq $args{Process}) {
      # print $line."\n";
      if (1) {
	my @list = split /\s+/, $line;
	push @pids, $list[1];
      } else {
	my $sample2 = "andrewdo  ";
	my $length2 = length($sample2);
	my $pid = $line;
	$pid =~ s/^.{$length2}//;
	# print Dumper($pid);
	if ($pid =~ /^(\d+)/) {
	  push @pids, $1;
	}
      }
    }
  }
  return \@pids;
}

sub KillProcesses {
  my %args = @_;
  my $pids = PidsForProcess
    (
     Process => $args{Process},
     Verbose => 1,
    );
  ApproveCommands
    (
     Commands => [
		  "kill -9 ".join(" ",@$pids),
		 ],
     Method => "parallel",
    );
}

1;
